class LengthEqualtyError(Exception):
    pass


class ImageEncodeError(Exception):
    pass


class MaxFileSizeExeeded(Exception):
    pass


class NotSupportedInputFile(Exception):
    pass

class InitializationError(Exception):
    pass

class CommandError(Exception):
    pass